#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# License: GNU General Public License v2

# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-06-29
#
# Check Point base inventory
#
# 2016-06-29: inventory Check Point Appliance
# 2018-03-05: added Patches, Deployment Agent Build
# 2018-03-07: added Licenses
# 2020-06-01: cleanup, prepared for cmk1.7x, rename from inv_checkpoint_svn to checkpoint_inv_base
# 2020-11-27: rewrite for CMK check API 1.0 (CMK 2.0)
# 2020-11-28: added available updates
# 2021-02-08: transferred license/support info to separate plugin
# 2021-02-22: code cleanup
# 2021-03-05: added hostlabel section
# 2021-03-18: fixed missing update agent snmp section.
# 2021-07-24: fixed parse function for empty string_table
# 2023-04-23: refactoring
#

from typing import List, NamedTuple, Optional
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    HostLabelGenerator,
    StringTable,
    InventoryResult,
)
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    Attributes,
    HostLabel,
    register,
    SNMPTree,
    startswith,
    all_of,
    any_of,
    equals,
)


class CheckpointInvBaseInfo(NamedTuple):
    serial_number: str
    product_name: str
    manufacturer: str
    series: str
    version: str
    build: str
    osname: str
    os_major_ver: str
    os_minor_ver: str


def parse_inv_checkpoint_base(string_table: List[StringTable]) -> Optional[List]:
    """
    >>> from pprint import pp
    >>> string_table = [[['5M7C043','Smart-1 5150','CheckPoint','Smart-1','R80.40','994000022','Gaia','3','10']]]
    >>> string_table.append([['1959']])
    >>> print(string_table)
    [[['5M7C043', 'Smart-1 5150', 'CheckPoint', 'Smart-1', 'R80.40', '994000022', 'Gaia', '3', '10']], [['1959']]]
    >>> pp(parse_inv_checkpoint_base(string_table))
    [(['hardware', 'system'], 'serial', '5M7C043'),
     (['hardware', 'system'], 'appliance_series', 'Smart-1'),
     (['hardware', 'system'], 'manufacturer', 'CheckPoint'),
     (['hardware', 'system'], 'product_name', 'Smart-1 5150'),
     (['software', 'check_point', 'os_info'], 'svn_version', 'R80.40'),
     (['software', 'check_point', 'os_info'], 'svn_build', '994000022'),
     (['software', 'check_point', 'os_info'], 'os_name', 'Gaia'),
     (['software', 'check_point', 'os_info'], 'deployment_agent_build', '1959'),
     (['software', 'check_point', 'os_info'], 'os_version', '3.10')]
    """
    try:
        base_info = CheckpointInvBaseInfo(*string_table[0][0])
    except IndexError:
        return

    try:
        update_agent_version = string_table[1][0][0]
    except IndexError:
        update_agent_version = ''

    section = []
    path = ['hardware', 'system']

    for key, value in [
        ('serial', base_info.serial_number),
        ('appliance_series', base_info.series),
        ('manufacturer', base_info.manufacturer),
        ('product_name', base_info.product_name),
    ]:
        if not value == '':
            section.append((path, key, value))

    path = ['software', 'check_point', 'os_info']

    for key, value in [
        ('svn_version', base_info.version),
        ('svn_build', base_info.build),
        ('os_name', base_info.osname),
        ('deployment_agent_build', update_agent_version),
        ('os_version', base_info.os_major_ver + '.' + base_info.os_minor_ver),
    ]:
        if not value == '':
            section.append((path, key, value))

    return section


def host_label_inv_checkpoint_base(section: List) -> HostLabelGenerator:
    for invPath, key, value in section:
        if key == 'appliance_series' and value.lower() == 'maestro':
            yield HostLabel('checkpoint/device_type', 'maestro_security_group')


def inventory_checkpoint_base(section: List) -> InventoryResult:
    for invPath, key, value in section:
        yield Attributes(
            path=invPath,
            inventory_attributes={key: value})


register.snmp_section(
    name='inv_checkpoint_base',
    parse_function=parse_inv_checkpoint_base,
    host_label_function=host_label_inv_checkpoint_base,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.6',  # CHECKPOINT-MIB::svn
            oids=[
                '16.3',  # svnApplianceSerialNumber
                '16.7',  # svnApplianceProductName
                '16.9',  # svnApplianceManufacturer
                '16.10',  # svnApplianceSeries
                '4.1',  # svnVersion
                '4.2',  # svnBuild
                '5.1',  # osName
                '5.2',  # osMajorVer
                '5.3',  # osMinorVer
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.6.20',  # CHECKPOINT-MIB::svnUpdatesInfo
            oids=[
                '1',  # svnUpdatesInfoBuild
            ]
        ),
    ],
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),  # sysObjectID == CheckPoint
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),  # sysObjectID == Linux
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),  # CheckPoint software installed
        )
    )
)

register.inventory_plugin(
    name='inv_checkpoint_base',
    inventory_function=inventory_checkpoint_base,
)
